#!/usr/bin/env python
# -*- coding: utf-8 -*-

#  Copyright 2020 Abdelkrime Aries <kariminfo0@gmail.com>
#
#  ---- AUTHORS ----
# 2020	Abdelkrime Aries <kariminfo0@gmail.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

from collections import OrderedDict

version = "0.1"
release = "0.1.0"

class NodeProcessor(object):

    def _pre_process(self, node):
        pass

    def _post_process(self, node):
        pass

    def _child_pre_process(self, node, child_key):
        return False

    def _child_post_process(self, node, child_key):
        return False

    def root_init(self, root):
        pass

    def root_final(self, root):
        pass

    def result(self):
        pass

    def process(self, node):
        children_keys = self._pre_process(node)
        for child_key in children_keys:
            if self._child_pre_process(node, child_key):
                next
            if self.process(node.children[child_key]):
                next
            if self._child_post_process(node, child_key):
                break
        self._post_process(node)

def_processor = NodeProcessor()

class Node(object):
    def __init__(self, value=0, label=""):
        self.label = label
        self.value = value
        self.children = OrderedDict()

    def append_child(self, label, node):
        self.children[label] = node
        return self


class Tree(object):
    """Generic tree representation

    Parameters
    ----------
    value : type
        the value of the root node
    label : type
        the label of the root node

    Attributes
    ----------
    root : Node
        The root node
    node_stack : array
        the stack for browsing nodes

    """

    def __init__(self, value=0, label=""):
        self.root = Node(value, label)
        self.node_stack = [self.root]

    def get_root(self):
        """Returns the root of the tree

        Parameters
        ----------


        Returns
        -------
        Node
            The root of the tree if it exists or None

        """
        return self.root

    def process(self, processor=def_processor):
        """Short summary.

        Parameters
        ----------
        processor : NodeProcessor
            the processor used to manipulate the tree starting from its root

        Returns
        -------
        type
            The result of processing, it can be None, a node, a string, etc.

        """
        processor.root_init(self.root)
        processor.process(self.root)
        processor.root_final(self.root)
        return processor.result()

    def add_child(self, label, node):
        """Adds a child to the current node.
        To browse the nodes use select_child and up

        Parameters
        ----------
        label : str or int
            The label of the arc
        node : Node
            The node to add as a child

        Returns
        -------
        Tree
            this object

        """
        current = self.node_stack[-1]
        current.append_child(label, node)
        return self

    def select_child(self, label):
        """Select a child node of the current one, using its label.
        Used to browsing the tree down.

        Parameters
        ----------
        label : str or int
            the label of the label of the arc

        Returns
        -------
        Tree
            this object

        """
        current = self.node_stack[-1]
        current = current.children[label]
        if current:
            self.node_stack.append(current)
        return self

    def up(self):
        """Select the parent of the current node.
        Used to browsing the tree up.

        Parameters
        ----------


        Returns
        -------
        type
            this object

        """
        if len(self.node_stack) > 1:
            self.node_stack.pop()
        return self

    def current_node(self):
        """Returns the current node

        Parameters
        ----------


        Returns
        -------
        Node
            The current node

        """
        return self.node_stack[-1]



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#  Copyright 2020 Abdelkrime Aries <kariminfo0@gmail.com>
#
#  ---- AUTHORS ----
# 2020	Abdelkrime Aries <kariminfo0@gmail.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

from . import NodeProcessor

class GraphvizProcessor(NodeProcessor):
    """Generates DOT representation of a tree"""

    def __fill_node(self, node):
        self.nbr += 1
        nid = 'N' + str(self.nbr)
        self.graph += nid + '[label="' + node.label + '"];\n'
        return nid

    def _pre_process(self, node):
        if len(self.father_id) == 0:
            self.father_id.append(self.__fill_node(node))
        return node.children.keys()

    def _child_pre_process(self, node, key):
        nid = self.__fill_node(node.children[key])
        self.graph += self.father_id[-1] + ' -> ' + nid + ' [label="' + key + '"];\n'
        self.father_id.append(nid)
        return False

    def _child_post_process(self, node, child_key):
        self.father_id.pop()
        return False

    def root_init(self, root):
        self.nbr = 0
        self.graph = "digraph Tree {\nnode [shape=box] ;\n"
        self.father_id = []

    def root_final(self, root):
        self.graph += "}"

    def result(self):
        return self.graph

graphviz_processor = GraphvizProcessor()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#  Copyright 2020 Abdelkrime Aries <kariminfo0@gmail.com>
#
#  ---- AUTHORS ----
# 2020	Abdelkrime Aries <kariminfo0@gmail.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

from . import NodeProcessor, Node

class GenerateProcessor(NodeProcessor):
    """Generates a tree given a representation using dictionaries

    Parameters
    ----------
    rep : dict
        A dictionary representing the tree

    Attributes
    ----------
    rep

    """

    def __init__(self, rep):
        self.rep = rep

    def __fill_node(self, node, rep):
        for val in rep:
            setattr(node, val, rep[val])

    def _pre_process(self, node):
        if not node.children:
            return []
        return node.children.keys()

    def _child_pre_process(self, node, key):
        child = Node()
        self.__fill_node(child, node.children[key])
        node.children[key] = child
        return False

    def _child_post_process(self, node, child_key):
        return False

    def root_init(self, root):
        self.__fill_node(root, self.rep)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#  Copyright 2020 Abdelkrime Aries <kariminfo0@gmail.com>
#
#  ---- AUTHORS ----
#  2020	Abdelkrime Aries <kariminfo0@gmail.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

from setuptools import setup

from os import path
import io

this_directory = path.abspath(path.dirname(__file__))
with io.open(path.join(this_directory, "README.md"), encoding="utf-8") as f:
    long_description = f.read()

setup(
    name = "shajara",
    version = "0.1.0",
    author = "Abdelkrime Aries",
    author_email = "kariminfo0@gmail.com",
    description = ("Manupilating trees"),
    license = "MIT",
    keywords = "trees",
    url = "https://github.com/kariminf/shajara",
    packages=["shajara"],
    #scripts=["exec/aruudy", "exec/aruudy_flask"],
    long_description=long_description,
    long_description_content_type="text/markdown",
    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "Intended Audience :: Education",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.6",
        "Topic :: Software Development :: Build Tools",
    ]
)
